// Copyright (c) 2020, The Loki Project
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
//    conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
//    of conditions and the following disclaimer in the documentation and/or other
//    materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors may be
//    used to endorse or promote products derived from this software without specific
//    prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pragma once

#include <set>
#include <unordered_set>

#include "container.h"

namespace serialization {

namespace detail {

    template <serializing Archive, typename Set>
    void serialize_set(Archive& ar, Set& set) {
        size_t cnt = set.size();
        auto arr = ar.begin_array(cnt);
        for (auto& v : set) {
            // We're serializing so this won't actually change v, despite casting away the
            // const, but the serialization code is a bit inflexible with const types.
            serialize_container_element(ar, const_cast<typename Set::value_type&>(v));
        }
    }

    template <deserializing Archive, typename Set>
    void serialize_set(Archive& ar, Set& set) {
        size_t cnt;
        auto arr = ar.begin_array(cnt);
        set.clear();
        for (size_t i = 0; i < cnt; i++) {
            typename Set::value_type v;
            serialize_container_element(ar, v);
            set.insert(std::move(v));
        }
    }

}  // namespace detail

template <class Archive, class T>
void serialize_value(Archive& ar, std::set<T>& v) {
    detail::serialize_set(ar, v);
}
template <class Archive, class T>
void serialize_value(Archive& ar, std::unordered_set<T>& v) {
    detail::serialize_set(ar, v);
}

}  // namespace serialization
